/*
 * main.cpp
 *
 *  Created on: Feb 9, 2020
 *      Author: ayoungblood
 */

#include "Game.hpp"
#include "../cjson/cJSON.h"
#include <string>
#include <iostream>
#include <fstream>
#include <sstream>
#include <boost/filesystem.hpp>

Game *game = nullptr;

int main(int argc, const char * argv[])
{
//  printf("Program has started, but not initialized yet.\n");
	const int FPS = 60;
	const int frameDelay = 1000 / FPS;

	Uint64 frameStart;
	int frameTime;

  boost::filesystem::path p{argv[0]};
  p = absolute(p).parent_path();
//  std::cout << "boost path: " << p << std::endl;

//      =============================
//      Load cJSON config.json file
//      =============================
// Starting with Error Checking    
    std::string configPath = "src/config/config.json";
    std::string bps = p.string();
    size_t charPos = bps.find("build");
    bps.erase(charPos,bps.length());
//    std::cout << "Trimmed Boost Path: " << bps << std::endl;
    std::string configurationPath = bps + configPath;
  //std::string configPath = "/Users/ayoungblood/Projects/KaijuSaveEarth/src/config/config.json";
//  /Users/ayoungblood/Projects/KaijuSaveEarth
//    std::string fullPath = p.c_str() + "/" + configPath;
    std::ifstream fin(configurationPath);
//    std::cout << "Attempting to load: " << configPath << std::endl;
      //  std::cout << "Current working Dir: " << std::filesystem::current_path() << std::endl;
    if(!fin.is_open()){
      std::cerr<<"ERROR: config.json not found or opened"<<std::endl;
    } else {
        printf("config.json loaded successfully.\n");
        
        std::ifstream jsonText(configurationPath);
        std::ostringstream tmp;
        tmp << jsonText.rdbuf();
        std::string json = tmp.str();
        cJSON * myJSON = cJSON_Parse(json.c_str());
        cJSON * windowName = cJSON_GetObjectItemCaseSensitive(myJSON, "WindowName");
        cJSON * windowSize = cJSON_GetObjectItem(myJSON, "WindowSize");
        int windowWidth = cJSON_GetObjectItem(windowSize, "w")->valueint;
        int windowHeight = cJSON_GetObjectItem(windowSize, "h")->valueint;
        int windowFS = cJSON_GetObjectItem(myJSON, "WindowFullScreen")->valueint;
        int globalScale = cJSON_GetObjectItem(myJSON, "GlobalScale")->valueint;
/*        int windowWidth = 64;
        int windowHeight = 64;
        int windowFS = 0;
        int globalScale = 10;*/
        bool isWindowFS;
        //std::string windowName = "KaijuSaveEarth";
        if (windowFS==0)
        {
        	isWindowFS = false;
        } else
        {
        	isWindowFS = true;
        }
        windowWidth = windowWidth*globalScale;
        windowHeight = windowHeight*globalScale;
	      game = new Game(bps);
	      game->init(windowName->valuestring, windowWidth, windowHeight, isWindowFS, globalScale);
//	      game->init(windowName.c_str(), windowWidth, windowHeight, isWindowFS, globalScale);
//	      cJSON memory management
        cJSON_Delete(myJSON);

	      while (game->running())
	      {
      		frameStart = SDL_GetTicks64();
      
      		game->handleEvents();
      		game->update();
      		game->render();
      
      		frameTime = SDL_GetTicks64() - frameStart;
      
      		if(frameDelay > frameTime)
      		{
      			SDL_Delay(frameDelay - frameTime);
      		}

	      }
	      game->clean();
    }
    
    if(fin.fail()){
        std::cout<<"config.json load failed"<<std::endl;
    } 
        
	return 0;
}
